use std::{collections::HashMap, io::Write};
use chrono::Datelike;
use chrono::NaiveDateTime;
use reqwest::Client;
use serde_json::json;
use serde::{Deserialize, Serialize};
use std::{
    env::{temp_dir, var},
    fs::File,
    io::Read,
    process::Command,
};
use colored::*;
use crate::error::CliError;

pub struct Settings {
    pub url: String,
    pub project: String,
    pub json: bool,
}

#[derive(Debug, Serialize, Deserialize, Clone)]
pub struct WorkPeriod {
    pub id: Option<i32>,
    pub project: String,
    pub start_time: NaiveDateTime,
    pub end_time: Option<NaiveDateTime>,
    pub description: Option<String>,
}

pub async fn start(settings: Settings, description: Option<&String>) -> Result<(), CliError> {
    let mut map = HashMap::new();
    map.insert("project", settings.project);
    if let Some(description) = description {
        map.insert("description", description.to_string());
    }
    let client = Client::new();
    let response = client.post(settings.url + "/api/tracking")
        .json(&map)
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(CliError::Server(response.text().await?));
    }
    Ok(())
}

pub async fn stop(settings: Settings, id: Option<&i32>) -> Result<(), CliError> {
    let mut map = HashMap::new();
    map.insert("project", settings.project);
    if let Some(id) = id {
        map.insert("id", format!("{}", id));
    }

    let client = Client::new();
    let response = client.delete(settings.url + "/api/tracking")
        .json(&map)
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(CliError::Server(response.text().await?));
    }
    Ok(())

}

fn parse_timestamp(timestamp: &str) -> String {
    let now = chrono::Local::now();
    let weekdays = ["monday", "tuesday", "wednesday", "thursday", "friday", "saturday", "sunday"];
    if timestamp == "today" {
        return now.format("%Y-%m-%dT00:00:00").to_string();
    } else if timestamp == "yesterday" {
        return (now - chrono::Duration::days(1)).format("%Y-%m-%dT00:00:00").to_string();
    } else {
        for (i, weekday) in weekdays.iter().enumerate() {
            if timestamp == *weekday {
                let days = (now.weekday().num_days_from_monday() as i64 - i as i64) % 7;
                return (now - chrono::Duration::days(days)).format("%Y-%m-%dT00:00:00").to_string();
            }
        }
    }
    return timestamp.to_string();
}

fn parse_periods(body: String) -> Result<Vec<WorkPeriod>, serde_json::Error> {
    let periods: Vec<WorkPeriod> = serde_json::from_str(&body)?;
    Ok(periods)
}

fn to_json(periods: Vec<WorkPeriod>) -> Result<String, serde_json::Error> {
    let json = json!(periods);
    Ok(serde_json::to_string_pretty(&json)?)
}

fn edit_periods(periods: Vec<WorkPeriod>) -> Result<Vec<WorkPeriod>, std::io::Error> {
    let content = to_json(periods)?;
    let editor = var("EDITOR").unwrap_or("vi".to_string());
    let mut file_path = temp_dir();
    file_path.push("Periods.json");
    let mut file = File::create(&file_path).expect("Could not create file");
    file.write_all(content.as_bytes())?;

    Command::new(editor)
        .arg(&file_path)
        .status()
        .expect("Something went wrong");

    let mut editable = String::new();
    File::open(file_path)
        .expect("Could not open file")
        .read_to_string(&mut editable)?;

    let periods: Vec<WorkPeriod> = serde_json::from_str(&editable).expect("Could not parse JSON");
    Ok(periods)
}

pub async fn update_period(settings: &Settings, period: WorkPeriod) -> Result<(), CliError> {
    let client = Client::new();
    let response = client.put(settings.url.to_string() + "/api/history/" + &period.id.unwrap().to_string())
        .json(&period)
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(CliError::Server(response.text().await?));
    }
    Ok(())
}

pub async fn add_period(settings: &Settings, period: &mut WorkPeriod) -> Result<(), CliError> {
    let client = Client::new();
    period.id = None;
    let response = client.post(settings.url.to_string() + "/api/history")
        .json(&period)
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(CliError::Server(response.text().await?));
    }
    Ok(())
}

pub async fn delete_period(settings: &Settings, id: i32) -> Result<(), CliError> {
    let client = Client::new();
    let response = client.delete(settings.url.to_string() + "/api/history/" + id.to_string().as_str())
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(CliError::Server(response.text().await?));
    }
    Ok(())
}

pub async fn edit(settings: Settings, since: Option<&String>, until: Option<&String>, num: Option<&String>) -> Result<(), CliError> {
    let mut params = vec![
        ("project", settings.project.to_owned()),
    ];
    if let Some(since) = since {
        params.push(("since", parse_timestamp(since)));
    }
    if let Some(until) = until {
        params.push(("until", parse_timestamp(until)));
    }
    if let Some(num) = num {
        params.push(("count", num.to_string()));
    }

    let client = Client::new();
    let url = reqwest::Url::parse_with_params((settings.url.to_owned() + "/api/history").as_str(), &params).unwrap();
    let response = client.get(url)
        .send()
        .await?;

    if !response.status().is_success() {
        return Err(CliError::Server(response.text().await?));
    } else {
        let body = response.text().await.unwrap();
        let periods = parse_periods(body).unwrap();
        let mut ids = periods.iter().map(|p| p.id.unwrap()).collect::<Vec<i32>>();
        let res = edit_periods(periods).unwrap();
        for period in res {
            let pos = ids.iter().position(|&x| x == period.id.unwrap());
            if let Some(pos) = pos {
                ids.remove(pos);
            } else {
                add_period(&settings, &mut period.clone()).await.unwrap();
            }
            update_period(&settings, period).await.unwrap();
        }
        for id in ids {
            delete_period(&settings, id).await.unwrap();
        }
    }
    Ok(())
}

#[derive(Debug, Serialize, Deserialize)]
pub struct Status {
    pub today: String,
    pub week: String,
    pub month: String,
    pub year: String,
    pub active: String,
}

fn format_bar(seconds: i64, max_value: i64, width: usize) -> String {
    const BLOCKS: &[&str] = &["▏", "▎", "▍", "▌", "▋", "▊", "▉", "█"];
    let ratio = (seconds as f64) / (max_value as f64);
    let full_blocks = (ratio * (width as f64)) as usize;
    let remainder = ((ratio * (width as f64) - full_blocks as f64) * 8.0) as usize;

    let mut bar = String::new();
    for _ in 0..full_blocks {
        bar.push_str(BLOCKS[7]);
    }
    if remainder > 0 && full_blocks < width {
        bar.push_str(BLOCKS[remainder - 1]);
    }
    while bar.len() < width {
        bar.push(' ');
    }

    // Color based on hours worked
    let hours = seconds as f64 / 3600.0;
    let colored_bar = match hours {
        h if h >= 8.0 => bar.green(),
        h if h >= 4.0 => bar.yellow(),
        _ => bar.red(),
    };

    format!("│{}", colored_bar)
}

pub async fn status(settings: Settings) -> Result<(), CliError> {
    // Get additional metrics from status endpoint
    let client = Client::new();
    let status_response = client.get(settings.url.to_string() + "/")
        .timeout(std::time::Duration::from_secs(10))
        .send()
        .await?;

    if !status_response.status().is_success() {
        return Err(CliError::Server(status_response.text().await?));
    }

    let status: Status = serde_json::from_str(&status_response.text().await?)?;

    // Compute current week boundaries (Monday to Sunday)
    let now = chrono::Local::now();
    let today = now.date_naive();
    let weekday = today.weekday().num_days_from_monday() as i64;
    let week_start_date = today - chrono::Duration::days(weekday);
    let week_start = week_start_date.and_hms(0, 0, 0);
    let week_end = week_start_date.and_hms(23, 59, 59) + chrono::Duration::days(6);

    // Build query URL for history API
    let query_params = [
        ("since", week_start.format("%Y-%m-%dT%H:%M:%S").to_string()),
        ("until", week_end.format("%Y-%m-%dT%H:%M:%S").to_string()),
        ("count", "100".to_string()),
        ("project", settings.project.clone()),
    ];
    let url = reqwest::Url::parse_with_params(&(settings.url + "/api/history"), &query_params).unwrap();

    let client = reqwest::Client::new();
    let response = client.get(url).send().await?;
    let body = response.text().await?;
    let periods = parse_periods(body).unwrap_or_default();

    // Check for active session first
    if let Some(active_period) = periods.iter().find(|p| p.end_time.is_none()) {
        let duration = now.naive_local().signed_duration_since(active_period.start_time);
        let duration_str = format_duration(duration.num_seconds());
        println!("\n{}", "Current Session".bold());
        println!("Active for: {} (started at {})",
            duration_str.yellow(),
            active_period.start_time.format("%H:%M").to_string().yellow()
        );
    }

    // Group durations by day (assume each period is within a single day)
    use std::collections::HashMap;
    let mut daily: HashMap<chrono::NaiveDate, i64> = HashMap::new();
    for period in &periods {
        let start = period.start_time;
        let end = period.end_time.unwrap_or(now.naive_local());
        let duration = (end - start).num_seconds();
        let day = start.date();
        *daily.entry(day).or_insert(0) += duration;
    }

    // Find maximum hours for scaling
    let max_seconds = daily.values().cloned().max().unwrap_or(8 * 3600);
    let max_seconds = std::cmp::max(max_seconds, 8 * 3600); // At least 8 hours for scale

    const BOX_WIDTH: usize = 50;
    const BAR_WIDTH: usize = 20;

    // Print weekly overview with visual bars
    println!("\n{}", "Weekly Work Overview".bold());
    println!("╭{}╮", "─".repeat(BOX_WIDTH));

    let mut total = 0;
    for i in 0..7 {
        let date = week_start_date + chrono::Duration::days(i);
        let seconds = daily.get(&date).cloned().unwrap_or(0);
        total += seconds;

        let day = date.format("%a").to_string().bold();
        let duration = format_duration(seconds);
        // Ensure the day and duration part is exactly 15 characters wide
        let day_str = format!("{:<3} {:<10}", day, duration);

        println!("{:<15} {:<width$}│",
            day_str,
            format_bar(seconds, max_seconds, BAR_WIDTH),
            width = BOX_WIDTH - 15
        );
    }

    println!("╰{}╯", "─".repeat(BOX_WIDTH));
    println!("\nTotal Worked: {}", format_duration(total).bold());

    let weekly_target = 39 * 3600;
    if total < weekly_target {
        println!("Remaining: {}", format_duration(weekly_target - total).red());
    } else {
        println!("Overtime: {}", format_duration(total - weekly_target).green());
    }

    // Print monthly and yearly totals
    println!("\n{}", "Extended Statistics".bold());
    println!("This Month: {}", status.month.cyan());
    println!("This Year:  {}", status.year.cyan());

    Ok(())
}

fn format_duration(seconds: i64) -> String {
    let hours = seconds / 3600;
    let minutes = (seconds % 3600) / 60;
    format!("{}h {}m", hours, minutes)
}
